"""
See http://trac.lliurex.net/pandora/browser/glusterfs/trunk/xlators/system/posix-acl/src/posix-acl-xattr.c?rev=6041
"""

import struct


READ          = (0x04)
WRITE         = (0x02)
EXECUTE       = (0x01)

UNDEFINED_TAG = (0x00)
USER_OBJ      = (0x01)
USER          = (0x02)
GROUP_OBJ     = (0x04)
GROUP         = (0x08)
MASK          = (0x10)
OTHER         = (0x20)

UNDEFINED_ID  = 2**32-1

perms = {
 (0x04):	'READ'          ,
 (0x02):	'WRITE'         ,
 (0x01):	'EXECUTE'       ,
}

tags = {
 (0x00):	'UNDEFINED_TAG' ,
 (0x01):	'USER_OBJ'      ,
 (0x02):	'USER'          ,
 (0x04):	'GROUP_OBJ'     ,
 (0x08):	'GROUP'         ,
 (0x10):	'MASK'          ,
 (0x20):	'OTHER'         ,
}


def read_acls(xattr):
	version = struct.unpack("<I", xattr[:4])
	acls = []
	for i in range(4, len(xattr), 8):
		acls.append(read_acl_entry(xattr[i:i+8]))
	return acls

def read_acl_entry(x):
	tag, perm, id = struct.unpack("<HHI", x)
	return Entry(tag, perm, id)

def write_acls(acls):
	xattr = struct.pack("<I", 2) #version 2
	for entry in acls:
		xattr += write_acl_entry(entry)
	return xattr

def write_acl_entry(entry):
	return struct.pack("<HHI", entry.tag, entry.perm, entry.id)

class Entry(object):
	def __init__(self, tag, perm, id = UNDEFINED_ID):
		self.tag = tag
		self.perm = perm
		self.id = id


