import rpclib
from os import mkdir, remove, statvfs
from sys import argv, exit
from copy import deepcopy
from base64 import b32encode
import pickle

class StorageServer(rpclib.RpcServer):
	def __init__(self):
		self.storepath = "file_storage"
		try:
			mkdir(self.storepath)
		except OSError:
			pass

	def _addr_to_path(self, addr):
		encoded = b32encode(addr)
		if len(encoded) > 250:
			raise Exception("addr too long", addr, encoded)
		path = self.storepath + '/' + encoded
		print path
		return path

	def rpc_read(self, addr):
		path = self._addr_to_path(addr)
		try:
			with open(path, 'r') as f:
				return pickle.load(f)
                except IOError:
                        return None

	def rpc_write(self, block):
		path = self._addr_to_path(block.addr)
		old_block = self.rpc_read(block.addr)
		if old_block and old_block.pubkey != block.pubkey:
			return False
		elif not block.checkSignature():
			return False
		try:
			with open(path, 'w') as f:
				pickle.dump(block, f)
		except IOError:
			print "Failed to write file '" + path + "' for addr '" + block.addr + "'"
			raise
		return True

	def rpc_delete(self, addr):
		path = self._addr_to_path(addr)
		try:
			remove(path)
		except IOError:
			print "Failed to remove file '" + path + "' for addr '" + block.addr + "'"
			raise

	def rpc_get_used_space(self):
		stat = statvfs(self.storepath)
		return (stat.f_blocks - stat.f_bavail) * stat.f_bsize

	def rpc_get_free_space(self):
		stat = statvfs(self.storepath)
		return stat.f_bavail * stat.f_bsize

if __name__ == '__main__':
	if len(argv) != 2:
		print 'usage: %s <port>' % argv[0]
		exit(1)
	(_, port) = argv
	port = int(port)
	s = StorageServer()
	s.run_socket_fork(port)
